#!/usr/bin/perl

#img2web by James Bromberger - web image publishing
#Copyright (C) 1999 James Bromberger
#
#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.



use strict;
use Image::Magick;
use Getopt::Long;
use Cwd;
use LWP::UserAgent;
use HTTP::Status;
use HTTP::Response;
use URI::URL;
use HTML::Parse;
use Sys::Hostname;
use vars qw($opt_upgrade $opt_version $opt_imagetail $opt_help %size $opt_debug $opt_scale $opt_columns $opt_smallImagePrefix $opt_indexname $opt_link $opt_sort $opt_templateSet $opt_indexHead $opt_indexTail $opt_imageHead $opt_indexTail $opt_title $opt_smallImagePrefix);

##############################
#
#
#  IMAGE to WEB - img2web
#
#
# A tool to publish a set of 
# JPEG images to the web with 
# full preview samples. 
# Contains lots of options to 
# customise the formatting.
#
# Web: http://www.ucc.gu.uwa.edu.au/~james/
# Email: james@ucc.gu.uwa.edu.au
#
#############################


##############################
#####  Location of programs we 
#####  depend on. You may need 
#####  to change these for 
#####  your system
##############################

my $sysconfigdir = "/etc/img2web/";
my $sysconfig = $sysconfigdir . "config";
my $usrconfigdir = "$ENV{'HOME'}/.img2web/";
my $usrconfig = $usrconfigdir . "config";
my $djpeg = "/usr/bin/djpeg";
my $cjpeg = "/usr/bin/cjpeg";
my $jpeginfo = "/usr/bin/jpeginfo";
my $version = "1.1b5";
my @upgradeURL = (
  "http://www.ucc.gu.uwa.edu.au/~james/programs/img2web/version.txt",
  "http://www.rcpt.to/~james/programs/img2web/version.txt");


##### No user servicable parts below here.


##############################
#####  Hash of options 
#####  containing defaults
##############################

my %options = (
  scale => "25 / 100",
  smallImagePrefix => "small-",
  indexName => "index.html",
  makeImageWebPage => 1,
  webPagePrefix => "preview-",
  pagelink => 1,
  indexHead => "",    
  indexTail => "",
  imageHead => "",
  imageTail => "",
  columns => "3",
  debug => "0",
  srcDir => ".",
  imgsort => "a",
  title => "Automatically generated by img2web v$version",
  );



PrintOptions() if ($options{"debug"} > 2);




##################################
##### Read in defaults from 
##### /etc/<name>/config
##################################


print "$0: testing if $sysconfig exists\n" if ($options{"debug"}>1);

  # If the system-wide config exists...
if (-e $sysconfig) {
  # The files exists, but is it readable?
  print "$0: testing if $sysconfig is readable\n" if ($options{"debug"} > 2);
  if (-r $sysconfig && -f $sysconfig && -T $sysconfig) {
    &ReadConfig ($sysconfig);
    }
  else {
    $! = 10;
    die "$0: Cannot open system configuration file $sysconfig.";
    }
  }
else {
  # There is no system config file...
  warn "$0: cannot find system config file $sysconfig. No big deal.\n";
  }



##################################
##### Read in the user's defaults
##################################

print "$0: testing if $usrconfig exists.\n" if ($options{"debug"}>1);

# If the users config file exists...
if (-e $usrconfig) {
  # The file exists, but is it readable?
  print "$0: testing to see if $usrconfig is readable.\n" if ($options{"debug"} > 2);
  if (-r $usrconfig && -f $usrconfig && -T $usrconfig) {
    &ReadConfig ($usrconfig);
    }
  else {
    $! = 11;
    die "$0: Cannot open user configuation file $usrconfig.\n";
    }
  }
  else {
    warn "$0: Cannot find user configuration file. No big deal.\n";
    }




##################################
##### Process the command line 
##### for these options
##################################


# If there are command line options...
if (@ARGV >0) {
  print "$0: reading command line options\n" if ($options{"debug"}>1);
  GetOptions("scale|s=s","smallimageprefix=s","indexname=s","debug=i", "link!", "sort:s", "templateset=s", "indexhead=s", "indextail=s", "imagehead=s", "imagetail=s", "columns|c=i", "title|t=s", "help|h", "version|v", "upgrade");


# Anything left on the command line must be a directory to process...

  if ($ARGV>0) {
    my $srcDir = shift @ARGV if $ARGV > 0;
    $options{'srcDir'} = $srcDir;
    if (!-d $options{'srcDir'}) {
      $! = 20;
      die "$0: Cannot write to this directory\n";
      }
    }

   # Sanity checking.....
  print "$0: check debug $opt_debug\n" if ($options{"debug"}>2);
  if (defined $opt_debug) {
    # Acceptable 0..5
    if (($opt_debug >=0) & ($opt_debug < 6)) {
      $options{'debug'} = $opt_debug;
      print "$0: debug set to $options{'debug'}.\n";
      }
    else {
      $! = 45;
      die "$0: cannot debug to level $opt_debug.\n";
      }
    }


  print "$0: check scale\n" if ($options{"debug"}>2);
  if (defined $opt_scale) {
    print "$0: scale is $opt_scale.\n" if ($options{'debug'} > 3);
    # Acceptable is scale is x% or x pixels.
    if ($opt_scale =~ /(\d{1,3})(%)/) {
      $options{'scale'} = "$1/100 ";
      print "$0: scale is $options{'scale'}\n" if ($options{'debug'} > 3);
      }
    elsif ($opt_scale =~ /(\d+)/) {
      $options{'scale'} = $1;
      }
    else {
      $! = 40;
      die "$0: cannot scale by this factor/size: $opt_scale.\n";
      }
    }



  print "$0: check columns\n" if ($options{"debug"}>2);
  # Acceptable 1 to 6
  if (defined $opt_columns) {
    if (($opt_columns <7) && ($opt_columns>0)) {
      $options{'columns'} = $opt_columns;
      }
    else {
      $! = 41;
      die "$0: cannot have $opt_columns columns (1..6).\n";
      }
    }


  print "$0: checking small image prefix\n" if ($options{"debug"}>2);
  if (defined $opt_smallImagePrefix) {
    # Acceptable any name that doesnt contain / (or non-printables?)
    if (!$opt_smallImagePrefix =~ /\//) {
      $options{'webPagePrefix'} = $opt_smallImagePrefix;
      }
    else {
      $! = 42;
      die "$0: cannot call prefix $opt_smallImagePrefix.\n";
      }
    }



  print "$0: checking index name\n" if ($options{"debug"}>2);
  if (defined $opt_indexname) {
    if (!$opt_indexname =~ /\//) {
      $options{'indexName'} = $opt_indexname;
      }
    else {
      $! = 43;
      die "$0: cannot have slashes in the index name.\n";
      }
    }



  print "$0: check link\n" if ($options{"debug"}>2);
  if (defined $opt_link) {
    $options{'pagelink'} = $opt_link;
    }



  print "$0: check sort\n" if ($options{"debug"}>2);
  if (defined $opt_sort) {
    # Acceptable a, d, filename
    if (($opt_sort == "a") | ($opt_sort == "s") | ($opt_sort == "d") | (-r $opt_sort)) {
      $options{'imgsort'} = $opt_sort;
      }
    else {
      $! = 44;
      die "$0: invalid sort $opt_sort.\n";
      }
    }



  print "$0: check template\n" if ($options{"debug"}>2);
  if (defined $opt_templateSet) {
    # Acceptable template name
    my $systemplate = $sysconfigdir . $opt_templateSet;

    my $systemplateindexhead= $systemplate . "indexhead.html";
    my $systemplateindextail= $systemplate . "indextail.html";
    my $systemplateimagehead= $systemplate . "imagehead.html";
    my $systemplateimagetail= $systemplate . "imagetail.html";

    my $usrtemplate = $usrconfigdir . $opt_templateSet;
    my $usrtemplateindexhead= $usrtemplate . "indexhead.html";
    my $usrtemplateindextail= $usrtemplate . "indextail.html";
    my $usrtemplateimagehead= $usrtemplate . "imagehead.html";
    my $usrtemplateimagetail= $usrtemplate . "imagetail.html";

    print "$0: Looking for $usrtemplateindexhead\n" if ($options{'debug'} > 3);
    print "$0: exists $usrtemplateindexhead\n" if ((-e $usrtemplateindexhead) && ($options{'debug'} > 3));
    print "$0: Readable $usrtemplateindexhead\n" if ((-r $usrtemplateindexhead) && ($options{'debug'} > 3));
    if ( -r $usrtemplateindexhead ) {
      print "$0: usertemplateindexhead is readable\n" if ($options{'debug'} > 3);
      $options{'template'} = $opt_templateSet;
      $options{'indexHead'} = $usrtemplateindexhead;
      }
    elsif (-r "$systemplateindexhead") {
      print "$0: systemplateindexhead is readable\n" if ($options{'debug'} > 3);
      $options{'template'} = $opt_templateSet;
      $options{'indexHead'} = $systemplateindexhead;
      }
    else {
      $! = 50;
      die "$0: cannot work out which template to use\n";
      }

    if (-r $usrtemplateindextail) {
      $options{'indexTail'} = $usrtemplateindextail;
      }
    elsif (-r $systemplateindextail) {
      $options{'indexTail'} = $systemplateindextail;
      }
    else {
      $! = 51;
      die "$0: cannot work out index tail template\n";
      }

    if (-r $usrtemplateimagehead) {
      $options{'imageHead'} = $usrtemplateimagehead;
      }
    elsif (-r $systemplateimagehead) {
      $options{'imageHead'} = $systemplateimagehead;
      }
    else {
      $! = 52;
      die "$0: cannot work out image head template\n";
      }

    if (-r $usrtemplateimagetail) {
      $options{'imageTail'} = $usrtemplateimagetail;
      }
    elsif (-r $systemplateimagetail) {
      $options{'imageTail'} = $systemplateimagetail;
      }
    else {
      $! = 53;
      die "$0: cannot work out image tail\n";
      }

    }

  print "$0: check index head\n" if ($options{"debug"}>2);

  if ((defined $opt_indexHead) & (-r $opt_indexHead)) {
    $options{'indexHead'} = $opt_indexHead;
    }
  print "$0: check index tail\n" if ($options{"debug"}>2);
  if ((defined $opt_indexTail) & (-r $opt_indexTail)) {
    $options{'indexTail'} = $opt_indexTail;
    }
  print "$0: check image head\n" if ($options{"debug"}>2);
  if ((defined $opt_imageHead) & (-r $opt_imageHead)) {
    $options{'imageHead'} = $opt_imageHead;
    }
  print "$0: check image tail\n" if ($options{"debug"}>2);
  if ((defined $opt_imageHead) & (-r $opt_imagetail)) {
    $options{'imageHead'} = $opt_imageHead;
    }

  print "$0: check title\n" if ($options{"debug"}>2);
  if (defined $opt_title) {
    $options{'title'} = $opt_title;
    }

  if (defined $opt_help) {
    &ShowHelp();
    exit 1;
    }

  if (defined $opt_version) {
    &ShowVersion();
    exit 2;
    }
  if (defined $opt_upgrade) {
    &CheckUpgrade();
    exit 0;
    }


  }




#################################
#### Find the images
#################################

my $oldwd = cwd();
chdir $options{"srcDir"};
my @images = imageList();

# Check they are readable. 

my $test;
foreach $test (@images) {
  die "$0: unable to read file $test. $!\n" unless -r $test;
  }


&PrintOptions() if $options{'debug'} > 1;


#############################################
#####  Now sort te image names according to the sort flag
#############################################

if ($options{'imgsort'} eq "a") {
  # Sort alphabetically
  print "$0: sorting alphabetically\n" if ($options{'debug'} > 2);
  @images = sort @images;
  }
elsif ($options{'imgsort'} eq "d") {
  # Sort on create date
  print "$0: sorting by file date\n" if ($options{'debug'} > 2);
  @images = sort sortFileDate @images;
  }
elsif ($options{'imgsort'} eq "s") {
  # Sort by file size
  print "$0: sorting by file size\n" if ($options{'debug'} > 2);
  @images = sort sortFileSize @images;
  }
elsif (-r $options{'imgsort'}) {
  print "$0: sorting by file $options{'sort'}\n" if ($options{'debug'} > 2);
  # Open the file, check each line not with a has mark
  # and arrange @images in that order, dropping files that
  # are not listed, and excluding files that do not exist.

  open INPUT, $options{'imgsort'} or die "$0: cannot open sorting file.\n";
  my $counter = 0;
  my @images2;
  while (<INPUT>) {
    next if /^#/;
    chomp;
    next unless (-r $_);
    $images2[$counter] =  $_;
    $counter = $counter + 1;
    }
  @images = @images2;
  }
else {
  die "$0: Cannot sort the images.\n";
  }

print "$0: images are @images.\n" if ($options{'debug'} > 1);
################################
#### Make the samples
################################

# Create the list of previous and next images
my (@prevImage, @nextImage);
my ($next, $last, $temp, $imgCounter);
$temp = "0";
$imgCounter = 0;

foreach $test (@images) {
  if ($imgCounter == 0) {
    $prevImage[0]= "0";
    $nextImage[0]= "0";
    }
  elsif ($imgCounter eq $#images) {
    $nextImage[$imgCounter] = "0";
    $prevImage[$imgCounter] = $temp;
    $nextImage[$imgCounter - 1] = $test;
    }
  else {
    $prevImage[$imgCounter] = $temp;
    $nextImage[$imgCounter - 1] = $test;
    }
  $imgCounter+=1;
  $temp = $test;
  }

&createSamples(@images);


# And put the samples into a page...

my $numImages = @images;
my $counter = 0;
my $image;

foreach $image (@images) {
  createImagePage($image, $prevImage[$counter], $nextImage[$counter++], $size{$image}[0], $size{$image}[1], $options{'indexHead'}, $options{'indexTail'});
  }



################################
#### Make the index
################################

# Dump the head and stuff
&backupFile($options{'indexName'});
&createIndex();
chdir $oldwd;
exit 0;

#
#
#
#
#
# End of program. Proceedures follow
#
#


sub ShowHelp {
# Print summary help
  print "-" x 75;
  print "\n";
  print "$0: Image to Web publisher\n\n";
  print "Version $version\n";
  print "Web: http://www.ucc.gu.uwa.edu.au/~james/\n";
  print "Usage:\n";
  print "$0 [-t string] [-smallimageprefix string]\n";
  print "\t[-indexname file] [-templateset string] [-indexhead file]\n";
  print "\t[-indextail file] [-imagehead file] [-imagetail file] [-debug n]\n";
  print "\t[-sort {a,d,file}] [-[no]link] [-columns n] [-help] [-version]\n\n";

  print "-t string\t\tTitle of all web pages generated is \"string\"\n";
  print "-smallimageprefix str\tPart of file name to prepend to preview images\n";
  print "-indexname file\t\tName the index page \"file\" \n";
  print "-templateset string\tThe set of 'head' and 'tail' files\n";
  print "-indexhead file\t\tThe 'head' for the index page\n";
  print "-indextail file\t\tThe 'tail' for the index page\n";
  print "-imagehead file\t\tThe 'head' for the image page\n";
  print "-imagetail file\t\tThe 'tail' for the image page\n";
  print "-debug n\t\tThe level of debugging output to be given\n";
  print "-sort {a,d,file}\tSort images Alpha., by Date, or by list in 'file'\n";
  print "-[no]link\t\tSpecify if each image should link to the next one\n";
  print "-columns n\t\tNumber of columsn in index\n";
  print "-help\t\t\tThis help page\n";
  print "-version\t\tVersion information of this program\n";
  print "-upgrade\t\tCheck on-line server for newer version\n\n";

  print "References to 'string' above may need to be quoted (eg, \"text text\").\n";
  print "-" x 75;
  print "\n";
  }

sub ShowVersion {
# Print version number
  print "$0 v$version\n\n";
  }


sub ReadConfig {
# Read in the config file we are passed as an file name!
  my $file = shift @_;
  print "$0: About to open config file $file..." if ($options{'debug'} > 2);
  open CONFIG, $file or die "$0: Cannot open config.\n";
  print "ok.\n" if ($options{'debug'} > 2);

  while (<CONFIG>) {
    next if /^#/;  # skip comments
    next if /^$/;  # skip blank lines

    # Now scan for a line that starts with a work in column 
    # one and is of the form:  foo=bar
    # optional space between equals, case insensative, one per line
    #/^(\w+)\s*=\s*(\w+)/i;
    /^(\w+)\s*=\s*(.+)\s*/i;
    my $var = $1;
    my $val = $2;

    print "$0: config file $file says $var = $val.\n" if ($options{'debug'} > 1);
    for ($var) {
      /columns/i  and do  {$options{'columns'} = $val if (($val > 0) & ($val < 6)); last; };
      /scale/i  and do  {$options{'scale'} = "$val"; last; } ;
      /smallImagePrefix/i  and do  {$options{'smallImagePrefix'} = $val if (!$val =~ /\//);last;};
      /webPagePrefix/i  and do  {$options{'webPagePrefix'} = $val if (!$val =~ /\//);last;};
      /indexName/i  and do  {$options{'indexName'} = $val if (!$val =~ /\//); last;};
      /makeWebPagePrefix/i  and do  {$options{'makeWebPagePrefix'} = $val;last;};
      /link/i  and do  {$options{'pagelink'} = $val if (($val =~/true/i) | ($val =~/false/i)) ;last;};
      /indexHead/i  and do  {$options{'indexHead'} = $val;last;};
      /indexTail/i  and do  {$options{'indexTail'} = $val; last;};
      /imageHead/i  and do  {$options{'imageHead'} = $val;last;};
      /imageTail/i  and do  {$options{'imageTail'} = $val;last;};
      /sort/i    and do  {$options{'imgsort'} = $val if (($val eq "a") | ($val eq "d") | (-r $val));last;};
      /title/i  and do  {$options{'title'} = $val;last;};
      }
    }
  close CONFIG;
  }

sub PrintOptions {
  print "$0: Printing options...\n";
  foreach $test (keys %options) {
    print "$0:   - $test = \"$options{$test}\".\n";
    }
  }


sub sortFileDate {
  my @fileA;
  my @fileB;
  @fileA = stat $a;
  @fileB = stat $b;

  $fileA[9] <=> $fileB[9];
  }

sub sortFileSize {
  my @fileA;
  my @fileB;
  @fileA = stat $a;
  @fileB = stat $b;

  $fileA[7] <=> $fileB[7];
  }


sub CheckUpgrade {
  my $ua = new LWP::UserAgent;
  $ua->agent("img2web/$version");
  my $code = 0;
  my $hostcounter = 0;
  my $path = "";
  my $host = hostname();
  my ($username, @other) = getpwuid $<;
  my $userinfo = "?user=$username&host=$host";
  my $body;
  ($options{'debug'} > 2) && print "$0: Checking for upgrade...\n";

  # Loop through all the known servers until we get an OK error code (200)
  for($hostcounter = 0; $hostcounter < $#upgradeURL && $code != 200; $hostcounter++) {
  ($options{'debug'} > 2) && print "$0: using $upgradeURL[$hostcounter]...\n";
  chomp $upgradeURL[$hostcounter];
  $path = "$upgradeURL[$hostcounter]" . $userinfo;
    # Proxy support
    if (defined $ENV{'http_proxy'}) {
      my $url = new URI::URL $path;
      my $scheme = $url -> scheme;
      $ua->agent($scheme, $ENV{'http_proxy'});
      }
    my $request = new HTTP::Request("GET", $path);
    my $response = $ua->request($request);
    $code = $response->code;
    $body = $response->content;
    ($options{'debug'} > 2) && print "$0: I got code $code, content \"$body\"\n";
    }
  if ($code == 200) {
    # We got one. Parse the output as:
    #    * First line is current available version
    #    * second and subsequent lines are comments

    my @lines = split /^/m, $body;
    my $exitcode;
    my $length = $#lines + 1;

    print "Currently: $version. ";
    if ($version < $lines[0]) {
      print "Upgrade available to version $lines[0]";
      $exitcode = 100;
      }
    elsif ($version > $lines[0]) {
      print "You have a beta test version";
      $exitcode = 101;
      }
    else {
      print "No upgrade available";
      $exitcode = 102;
      }
    print ".\n";
    print "\n" if ($length > 1);
    for (my $counter = 1; $counter < $length; $counter++) {
      print $lines[$counter];
      }
    exit $exitcode;
    }
  else {
    warn "Unable to contact upgrade servers.\n";
    }
  }



sub imageList {
  my $dirToCheck = @_;

  opendir THISDIR, $options{"srcDir"} or die "$0: Can't open directory: $!";
  # Get a list of images that are not previously generated preview versions
  my @imageList = grep /(\.jpg$)|(\.gif$)/i && $` !~ /$options{'smallImagePrefix'}/, readdir THISDIR;
  closedir THISDIR;
  return @imageList;
  }



sub createSamples {
  my @originals = @_;
  my $test;
  foreach $test (@originals) {
    my $origImg = new Image::Magick;
    $origImg->Read($test);
    my ($bigwidth, $bigheight) = $origImg->Get('columns', 'rows');
    $options{'scale'} = eval $options{'scale'};
    my $width = $bigwidth * $options{'scale'};
    my $height= $bigheight * $options{'scale'};

    # does the image already exist?
    my $oldImg = new Image::Magick;
    my($curW, $curH, $curS, $curF)= split(',', $oldImg->Ping("$options{'smallImagePrefix'}$test"));

    @size{$test} = [ ($bigwidth, $bigheight, $width, $height) ];
    print "$0: Size of $test is \"$size{$test}[0]\" x \"$size{$test}[1]\"\n";
    if ($curW != $width){
      $origImg->Scale(width=>$width, height=>$height);
      print "$0: $test is $bigwidth x $bigheight - scaling to $options{'scale'} - $width, $height\n" if ($options{'debug'} > 2);
      print "$0: Writing $options{'smallImagePrefix'}$test\n";
      $origImg->Write("$options{'smallImagePrefix'}$test");
      }
    else {
    	print "$0: Scaled image already exists ($curW x $curH)!\n";
      }
    }
  }


sub createImagePage {
  my $img = $_[0];
  my $prevImage = $_[1];
  my $nextImage = $_[2];
  my $width = $_[3];
  my $height = $_[4];
  my $top = $_[5];
  my $tail = $_[6];
  my $now = scalar gmtime;

  # What is the name of the page to create?
  open OUTPUT, ">" . $options{"webPagePrefix"} . $img . ".html"  or die "canno
t open output";
  if ( -r $options{"imageHead"} && -r $options{"imageTail"}) {
    open INPUT, $options{"imageHead"} or die "Cannot open image top file";
    while (<INPUT>) {
      if (/<\/HEAD>/) {
        print OUTPUT "<META NAME=\"Generated\" CONTENT=\"James Bromberger's img2web script\">\n";
        print OUTPUT "<META NAME=\"Created\" CONTENT=\"$now UTC\">\n";
        print OUTPUT "<TITLE>$options{'title'}</TITLE>\n";
        }
      print OUTPUT;
      }
    close INPUT;
    }
  else {
    # There is no starting template.
    print OUTPUT "<HTML LANG=\"En\">\n";
    print OUTPUT "<HEAD>\n";
    print OUTPUT "<TITLE>$options{'title'}<\/TITLE>\n";
    print OUTPUT "<META NAME=\"Generated\" CONTENT=\"James Bromberger's img2web script\">\n";
    print OUTPUT "<META NAME=\"Created\" CONTENT=\"$now UTC\">\n";
    print OUTPUT "</HEAD>\n";
    print OUTPUT "<BODY BGCOLOR=\"#FFFFFF\">\n";
    }


  # Now put in the image! ;)
  print OUTPUT "<IMG SRC=\"$img\" WIDTH=\"$width\" HEIGHT=\"$height\" ALT=\"\">\n";

  # And now for the document tail...
  if ( -r $options{"imageHead"} && -r $options{"imageTail"}) {
    open INPUT, $options{'imageTail'} or die "$0: Cannot open $options{'imageTail'}";
    my $inComment;
    $inComment = 0;
    while (<INPUT>) {
      my $line = $_;
      if (/<!-- (.*) -->/ && $options{'pagelink'}) {
        my @lineparts = split (/(<!--|-->)/);
        foreach my $bit (@lineparts) {
          print "$0: Checking split - \"$bit\"\n" if ($options{'debug'} > 3);
          if ($bit =~ /<!--/) {
            $inComment++;
            }
          elsif ($bit =~ /-->/) {
            $inComment--;
            }
          elsif ($inComment > 0) {
            print "$0: in comment - \"$bit\"\n" if ($options{'debug'} > 3);
            if ($bit =~ /img2web/i) {
              if ( $prevImage ne "0") {
                print OUTPUT "<A HREF=\"$options{'webPagePrefix'}" . $prevImage . ".html\">" if ($bit =~ /previous link start/i);
                }
              print OUTPUT "</A>" if ($bit =~ /previous link end/i);
              if ( $nextImage ne "0") {
                print OUTPUT "<A HREF=\"$options{'webPagePrefix'}" . $nextImage . ".html\">" if ($bit =~ /next link start/i);
                }
              print OUTPUT "</A>" if ($bit =~ /next link end/i);
              print OUTPUT "<A HREF=\"$options{'indexName'}\">" if ($bit =~ /index link start/i);
              print OUTPUT "</A>" if ($bit =~ /index link end/i);
              }
            else {
              print OUTPUT "<!-- " . $bit . " -->";
              }
            }
          else {
            print OUTPUT $bit;
            }
          }
        }
      else {
        print OUTPUT;
        }
      }
    close INPUT;
    }
  else {
    # There is no tail template
    if ($options{'pagelink'}) {
      # We need to link to either the last image in 
      # the list @images or the next one, and look out 
      # for the first and last case
      print "$0: Last one $prevImage next is $nextImage and counter $counter\n" if ($options{'debug'} > 3);
      print OUTPUT "<P>\n";
      print OUTPUT "<A HREF=\"$options{'indexName'}\">Image Samples</A>\n";
      if ( $prevImage ne "0") {
        print OUTPUT "<A HREF=\"$options{'webPagePrefix'}" . $prevImage . ".html\">Last</A> | ";
        }
      else {
        print OUTPUT "Last | ";
        }


      if ( $nextImage ne "0") {
        print OUTPUT "<A HREF=\"$options{'webPagePrefix'}" . $nextImage . ".html\">Next</A>\n";
        }
      else {
        print OUTPUT "Next\n";
        }
      print OUTPUT "</P>\n\n";

      # dump end body
      print OUTPUT "</BODY>\n</HTML>\n";
      }
    }
  close OUTPUT
  }



sub backupFile {
  my $file = @_[0];
  if (-x $file) {
    rename $file, "$file.0" or die "Cannot backup $file";
    }
  }



sub createIndex {
  my @list = @images;
  my %information = %size;

  my $haveTable = 0;

  open OUTPUT, ">" . $options{'indexName'} or die "$0: cannot open output index\n";
  if (-r $options{'indexHead'}) {
    open INPUT, $options{'indexHead'} or die "$0: Cannot open input file\n";
    my $haveTitle = 0;
    print "$0: checking index head tmplate\n" if $options{'debug'} > 2;
    while (<INPUT>) {
      if (/<TITLE>/i) {
        # We found a title tag in our template? Use it?
        $haveTitle = 1;
        }
      if (/<\/HEAD>/i) {
        print OUTPUT "<META NAME=\"Generated\" CONTENT=\"James Bromberger's jpg2web script\">\n";
        print OUTPUT "<TITLE>$options{'title'}</TITLE>\n" unless $haveTitle == 1;
        }
      elsif (/<TABLE[^>]*?CLASS\w?=\w?.ImageTable./i) {
        $haveTable = 1;
        }
      print OUTPUT;
      }
    close INPUT;
    }
  else {
    # There is no readable output file head...
    print "$0: no template - dumping default\n" if $options{'debug'} > 2;
    print OUTPUT "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0 Transitional//EN\" \"http://www.w3.org/TR/REC-html40/loose.dtd\">\n";
    print OUTPUT "<HTML LANG=\"EN\">\n";
    print OUTPUT "<HEAD>\n";
    print OUTPUT "<META NAME=\"Generated\" CONTENT=\"James Bromberger's jpg2web script\">\n";
    print OUTPUT "<TITLE>$options{'title'}</TITLE>\n";
    print OUTPUT "</HEAD><BODY BGCOLOR=\"#FFFFFF\">\n";
    }

  if (!$haveTable) {
    print OUTPUT "<!-- Start of automatically inserted table by $0 -->\n";
    print OUTPUT "<TABLE BORDER=0 CLASS=\"ImageTable\">\n<TR>\n";
    }

  my $counter = 0;
  foreach $image (@images ) {
    print "$0: putting in $image\n" if $options{'debug'} > 4;
    print OUTPUT "<TD ALIGN=\"CENTER\">\n";
    print OUTPUT "<A HREF=\"$options{'webPagePrefix'}$image.html\">";
    print OUTPUT "<IMG SRC=\"$options{'smallImagePrefix'}$image\" HEIGHT=\"$size{$image}[3]\" WIDTH=\"$size{$image}[2]\" ALT=\"\"></A>\n";
    print OUTPUT "</TD>\n";
    $counter = $counter + 1;
    print OUTPUT "\</TR\>\n" if (($counter % $options{'columns'} == 0) | ($counter >= $numImages));
    print OUTPUT "\<TR>\n" if (($counter % $options{'columns'} == 0) & ($counter != $numImages));
   }

  if (!$haveTable) {
    print OUTPUT "</TABLE>\n";
    }
  if (-r $options{'indexTail'}) {
    open INPUT, $options{'indexTail'} or die "Cannot open index tail";
    while (<INPUT>) {
      print OUTPUT;
      }
    close INPUT;
    }
  else {
    print OUTPUT "</BODY>\n</HTML>\n";
    close OUTPUT;
    }
  }
